@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2009 Laszlo Molnar
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -51,8 +51,9 @@ void show_head(void)
     fg = con_fg(f,FG_GREEN);
     con_fprintf(f,
                 "                       Ultimate Packer for eXecutables\n"
-                "                          Copyright (C) 1996 - 2009\n"
+                "                          Copyright (C) 1996 - %s\n"
                 "UPX %-10s  Markus Oberhumer, Laszlo Molnar & John Reiser  %14s\n\n",
+                UPX_VERSION_YEAR,
 #if (ACC_OS_DOS16 || ACC_OS_DOS32)
                 V("d"),
 #elif (ACC_OS_WIN16 || ACC_OS_WIN32 || ACC_OS_WIN64)
@@ -113,6 +114,7 @@ struct PackerNames
     {
         PackerNames *self = (PackerNames *) user;
         self->add(p);
+        delete p; p = NULL;
         return NULL;
     }
     static int __acc_cdecl_qsort cmp_fname(const void *a, const void *b) {
@@ -398,10 +400,10 @@ void show_version(int x)
     if (v != NULL && v[0])
         fprintf(fp, "LZMA SDK version %s\n", v);
 #endif
-    fprintf(fp, "Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer\n");
-    fprintf(fp, "Copyright (C) 1996-2009 Laszlo Molnar\n");
-    fprintf(fp, "Copyright (C) 2000-2009 John F. Reiser\n");
-    fprintf(fp, "Copyright (C) 2002-2009 Jens Medoch\n");
+    fprintf(fp, "Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer\n");
+    fprintf(fp, "Copyright (C) 1996-2010 Laszlo Molnar\n");
+    fprintf(fp, "Copyright (C) 2000-2010 John F. Reiser\n");
+    fprintf(fp, "Copyright (C) 2002-2010 Jens Medoch\n");
 #if defined(WITH_ZLIB)
     fprintf(fp, "Copyright (C) 1995" "-2005 Jean-loup Gailly and Mark Adler\n");
 #endif
