@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -22,7 +22,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Markus F.X.J. Oberhumer              Laszlo Molnar
-   <mfx@users.sourceforge.net>          <ml1050@users.sourceforge.net>
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -633,6 +633,17 @@ static int do_option(int optc, const char *arg)
         opt->debug.disable_random_id = true;
         break;
 
+    // mp (meta)
+    case 501:
+        getoptvar(&opt->mp_compress_task, 1, 999999, arg);
+        break;
+    case 502:
+        opt->mp_query_format = true;
+        break;
+    case 503:
+        opt->mp_query_num_tasks = true;
+        break;
+
     // misc
     case 512:
         opt->console = CON_FILE;
@@ -870,7 +881,10 @@ static int do_option(int optc, const char *arg)
         opt->ps1_exe.do_8bit = true;
         break;
     case 673:
-        opt->ps1_exe.do_8mb = false;
+        opt->ps1_exe.do_8mib = false;
+        break;
+    case 674:
+        opt->o_unix.unmap_all_pages = true;  // val ?
         break;
 
     case '\0':
@@ -1021,6 +1035,7 @@ static const struct mfx_option longopts[] =
     {"netbsd",           0x10, 0, 668},
     {"OpenBSD",          0x10, 0, 669},
     {"openbsd",          0x10, 0, 669},
+    {"unmap-all-pages",  0x10, 0, 674},     // linux /proc/self/exe vanishes
     // watcom/le
     {"le",               0x10, 0, 620},     // produce LE output
     // win32/pe
@@ -1034,8 +1049,14 @@ static const struct mfx_option longopts[] =
     {"boot-only",        0x10, 0, 670},
     {"no-align",         0x10, 0, 671},
     {"8-bit",            0x10, 0, 672},
+    {"8mib-ram",         0x10, 0, 673},
     {"8mb-ram",          0x10, 0, 673},
 
+    // mp (meta) options
+    {"mp-compress-task",        0x31, 0, 501},
+    {"mp-query-format",         0x10, 0, 502},
+    {"mp-query-num-tasks",      0x10, 0, 503},
+
     { NULL, 0, NULL, 0 }
 };
 
@@ -1106,6 +1127,13 @@ static const struct mfx_option longopts[] =
     // compression settings
     {"exact",            0x10, 0, 525},     // user requires byte-identical decompression
 
+    // compression method
+    {"nrv2b",            0x10, 0, 702},     // --nrv2b
+    {"nrv2d",            0x10, 0, 704},     // --nrv2d
+    {"nrv2e",            0x10, 0, 705},     // --nrv2e
+    {"lzma",             0x10, 0, 721},     // --lzma
+    {"no-lzma",          0x10, 0, 722},     // (disable all_methods_use_lzma)
+    // compression settings
     // compression runtime parameters
 
     // win32/pe
@@ -1233,8 +1261,8 @@ template <class T> struct TestBELE {
 static bool test(void)
 {
     COMPILE_TIME_ASSERT_ALIGNED1(T)
-    struct test1_t { char a; T b; } __attribute_packed;
-    struct test2_t { char a; T b[3]; } __attribute_packed;
+    __packed_struct(test1_t) char a; T b;    __packed_struct_end()
+    __packed_struct(test2_t) char a; T b[3]; __packed_struct_end()
     test1_t t1[7]; UNUSED(t1); test2_t t2[7]; UNUSED(t2);
     COMPILE_TIME_ASSERT(sizeof(test1_t) == 1 + sizeof(T))
     COMPILE_TIME_ASSERT_ALIGNED1(test1_t)
@@ -1246,7 +1274,11 @@ static bool test(void)
     COMPILE_TIME_ASSERT(__acc_alignof(t1) == 1)
     COMPILE_TIME_ASSERT(__acc_alignof(t2) == 1)
 #endif
-#if 1 && !defined(xUPX_OFFICIAL_BUILD)
+#if 1 && (ACC_CC_WATCOMC)
+    test1_t t11; COMPILE_TIME_ASSERT(sizeof(t11.a) <= sizeof(t11.b))
+    test2_t t22; COMPILE_TIME_ASSERT(sizeof(t22.a) <= sizeof(t22.b))
+#endif
+#if 1 && !defined(UPX_OFFICIAL_BUILD)
     T allbits; allbits = 0; allbits -= 1;
     //++allbits; allbits++; --allbits; allbits--;
     T v1; v1 = 1; v1 *= 2; v1 -= 1;
@@ -1319,27 +1351,40 @@ void upx_sanity_check(void)
     assert(TestBELE<BE32>::test());
     assert(TestBELE<BE64>::test());
     {
-    static const unsigned char dd[32] = { 0, 0, 0, 0, 0, 0, 0,
-         0xff, 0xfe, 0xfd, 0xfc, 0xfb, 0xfa, 0xf9, 0xf8,
-         0, 0, 0, 0,
-         0x7f, 0x7e, 0x7d, 0x7c, 0x7b, 0x7a, 0x79, 0x78,
+    static const unsigned char dd[32]
+#if 1 && (ACC_CC_GNUC || ACC_CC_INTELC || ACC_CC_PATHSCALE) && defined(__ELF__)
+        __attribute__((__aligned__(16)))
+#endif
+        = { 0, 0, 0, 0, 0, 0, 0,
+        0xff, 0xfe, 0xfd, 0xfc, 0xfb, 0xfa, 0xf9, 0xf8,
+        0, 0, 0, 0,
+        0x7f, 0x7e, 0x7d, 0x7c, 0x7b, 0x7a, 0x79, 0x78,
     0, 0, 0, 0, 0 };
     const unsigned char *d;
+    const N_BELE_RTP::AbstractPolicy *bele;
     d = dd + 7;
     assert(upx_adler32(d, 4) == 0x09f003f7);
     assert(upx_adler32(d, 4, 0) == 0x09ec03f6);
     assert(upx_adler32(d, 4, 1) == 0x09f003f7);
+    bele = &N_BELE_RTP::be_policy;
     assert(get_be16(d) == 0xfffe);
+    assert(bele->get16(d) == 0xfffe);
     assert(get_be16_signed(d) == -2);
     assert(get_be24(d) == 0xfffefd);
+    assert(bele->get24(d) == 0xfffefd);
     assert(get_be24_signed(d) == -259);
     assert(get_be32(d) == 0xfffefdfc);
+    assert(bele->get32(d) == 0xfffefdfc);
     assert(get_be32_signed(d) == -66052);
+    bele = &N_BELE_RTP::le_policy;
     assert(get_le16(d) == 0xfeff);
+    assert(bele->get16(d) == 0xfeff);
     assert(get_le16_signed(d) == -257);
     assert(get_le24(d) == 0xfdfeff);
+    assert(bele->get24(d) == 0xfdfeff);
     assert(get_le24_signed(d) == -131329);
     assert(get_le32(d) == 0xfcfdfeff);
+    assert(bele->get32(d) == 0xfcfdfeff);
     assert(get_le32_signed(d) == -50462977);
     assert(get_le64_signed(d) == ACC_INT64_C(-506097522914230529));
     assert(find_be16(d, 2, 0xfffe) == 0);
