@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2009 Laszlo Molnar
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -57,7 +57,7 @@ void printClearLine(FILE *f)
     fflush(stdout); fflush(stderr);
     if (f == NULL)
         f = stdout;
-    con_fprintf(f,clear_line_msg);
+    con_fprintf(f, "%s", clear_line_msg);
     fflush(f);
     printSetNl(0);
 }
