@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -21,8 +21,8 @@
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-   Markus F.X.J. Oberhumer   Laszlo Molnar
-   markus@oberhumer.com      ml1050@users.sourceforge.net
+   Markus F.X.J. Oberhumer              Laszlo Molnar
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -74,8 +74,7 @@ protected:
     virtual Linker* newLinker() const;
     void addLoaderEpilogue(int flag);
 
-    struct exe_header_t
-    {
+    __packed_struct(exe_header_t)
         LE16 ident;
         LE16 m512;
         LE16 p512;
@@ -91,8 +90,7 @@ protected:
         LE16 relocoffs;
         char __[2];             // overlnum
         LE32 firstreloc;
-    }
-    __attribute_packed;
+    __packed_struct_end()
 
     exe_header_t ih, oh;
 
