@@ -2,9 +2,9 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
-   Copyright (C) 2000-2007 John F. Reiser
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Laszlo Molnar
+   Copyright (C) 2000-2010 John F. Reiser
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -23,7 +23,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Markus F.X.J. Oberhumer              Laszlo Molnar
-   <mfx@users.sourceforge.net>          <ml1050@users.sourceforge.net>
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
 
    John F. Reiser
    <jreiser@users.sourceforge.net>
@@ -61,9 +61,14 @@ protected:
     ) = 0;
     virtual void defineSymbols(Filter const *);
     virtual void addStubEntrySections(Filter const *);
-    virtual void unpack(OutputFile *fo) = 0;
+    virtual void unpack(OutputFile *fo);
+
+    virtual unsigned elf_get_offset_from_address(acc_uint64l_t) const = 0;
+    virtual void const *elf_find_dynamic(unsigned) const = 0;
+    virtual acc_uint64l_t elf_unsigned_dynamic(unsigned) const = 0;
 
 protected:
+    unsigned e_phnum;       /* Program header table entry count */
     char       *file_image;       // if ET_DYN investigation
     char const *dynstr;   // from DT_STRTAB
 
@@ -72,6 +77,10 @@ protected:
     unsigned sz_pack2;  // after pack2(), before loader
     unsigned lg2_page;  // log2(PAGE_SIZE)
     unsigned page_size;  // 1u<<lg2_page
+    unsigned xct_off;  // shared library: file offset of SHT_EXECINSTR
+    unsigned hatch_off;  // file offset of escape hatch
+    acc_uint64l_t load_va;  // PT_LOAD[0].p_vaddr
+    acc_uint64l_t xct_va;  // minimum SHT_EXECINSTR virtual address
 
     unsigned short e_machine;
     unsigned char ei_class;
@@ -121,39 +130,48 @@ protected:
     virtual off_t getbase(const Elf32_Phdr *phdr, int e_phnum) const;
 
     static unsigned elf_hash(char const *) /*const*/;
-    virtual void const *elf_find_dynamic(unsigned) const;
+    static unsigned gnu_hash(char const *) /*const*/;
     virtual Elf32_Sym const *elf_lookup(char const *) const;
-    virtual unsigned elf_get_offset_from_address(unsigned) const;
+    virtual unsigned elf_get_offset_from_address(acc_uint64l_t) const;
+    Elf32_Shdr const *elf_find_section_name(char const *) const;
+    Elf32_Shdr const *elf_find_section_type(unsigned) const;
+    void const *elf_find_dynamic(unsigned) const;
+    acc_uint64l_t elf_unsigned_dynamic(unsigned) const;
 
 protected:
     Elf32_Ehdr  ehdri; // from input file
     Elf32_Phdr *phdri; // for  input file
+    Elf32_Shdr const *shdri; // from input file
     unsigned page_mask;  // AND clears the offset-within-page
 
     Elf32_Dyn    const *dynseg;   // from PT_DYNAMIC
     unsigned int const *hashtab;  // from DT_HASH
+    unsigned int const *gashtab;  // from DT_GNU_HASH
     Elf32_Sym    const *dynsym;   // from DT_SYMTAB
+    char const *shstrtab;   // via Elf32_Shdr
+    int n_elf_shnum;  // via e_shnum
 
-    struct cprElfHdr1 {
+    Elf32_Shdr const *sec_strndx;
+    Elf32_Shdr const *sec_dynsym;
+    Elf32_Shdr const *sec_dynstr;
+
+    __packed_struct(cprElfHdr1)
         Elf32_Ehdr ehdr;
         Elf32_Phdr phdr[1];
         l_info linfo;
-    }
-    __attribute_packed;
+    __packed_struct_end()
 
-    struct cprElfHdr2 {
+    __packed_struct(cprElfHdr2)
         Elf32_Ehdr ehdr;
         Elf32_Phdr phdr[2];
         l_info linfo;
-    }
-    __attribute_packed;
+    __packed_struct_end()
 
-    struct cprElfHdr3 {
+    __packed_struct(cprElfHdr3)
         Elf32_Ehdr ehdr;
         Elf32_Phdr phdr[3];
         l_info linfo;
-    }
-    __attribute_packed;
+    __packed_struct_end()
 
     cprElfHdr3 elfout;
 
@@ -211,31 +229,46 @@ protected:
     virtual unsigned find_LOAD_gap(Elf64_Phdr const *const phdri, unsigned const k,
         unsigned const e_phnum);
 
+    virtual unsigned elf_get_offset_from_address(acc_uint64l_t) const;
+    Elf64_Shdr const *elf_find_section_name(char const *) const;
+    Elf64_Shdr const *elf_find_section_type(unsigned) const;
+    void const *elf_find_dynamic(unsigned) const;
+    acc_uint64l_t elf_unsigned_dynamic(unsigned) const;
+
 protected:
     Elf64_Ehdr  ehdri; // from input file
     Elf64_Phdr *phdri; // for  input file
+    Elf64_Shdr const *shdri; // from input file
     acc_uint64l_t page_mask;  // AND clears the offset-within-page
 
-    struct cprElfHdr1 {
+    Elf64_Dyn    const *dynseg;   // from PT_DYNAMIC
+    unsigned int const *hashtab;  // from DT_HASH
+    unsigned int const *gashtab;  // from DT_GNU_HASH
+    Elf64_Sym    const *dynsym;   // from DT_SYMTAB
+    char const *shstrtab;   // via Elf64_Shdr
+    int n_elf_shnum;  // via e_shnum
+
+    Elf64_Shdr const *sec_strndx;
+    Elf64_Shdr const *sec_dynsym;
+    Elf64_Shdr const *sec_dynstr;
+
+    __packed_struct(cprElfHdr1)
         Elf64_Ehdr ehdr;
         Elf64_Phdr phdr[1];
         l_info linfo;
-    }
-    __attribute_packed;
+    __packed_struct_end()
 
-    struct cprElfHdr2 {
+    __packed_struct(cprElfHdr2)
         Elf64_Ehdr ehdr;
         Elf64_Phdr phdr[2];
         l_info linfo;
-    }
-    __attribute_packed;
+    __packed_struct_end()
 
-    struct cprElfHdr3 {
+    __packed_struct(cprElfHdr3)
         Elf64_Ehdr ehdr;
         Elf64_Phdr phdr[3];
         l_info linfo;
-    }
-    __attribute_packed;
+    __packed_struct_end()
 
     cprElfHdr3 elfout;
 
@@ -253,44 +286,33 @@ class PackLinuxElf32Be : public PackLinuxElf32
 {
     typedef PackLinuxElf32 super;
 protected:
-    PackLinuxElf32Be(InputFile *f) : super(f) { bele = &N_BELE_RTP::be_policy; }
-
-    virtual acc_uint64l_t get_native64(const void *b) const { return get_be64(b); }
-    virtual unsigned get_native32(const void *b) const { return get_be32(b); }
-    virtual unsigned get_native16(const void *b) const { return get_be16(b); }
-    virtual void set_native64(void *b, acc_uint64l_t v) const { set_be64(b, v); }
-    virtual void set_native32(void *b, unsigned v) const { set_be32(b, v); }
-    virtual void set_native16(void *b, unsigned v) const { set_be16(b, v); }
+    PackLinuxElf32Be(InputFile *f) : super(f) {
+        bele = &N_BELE_RTP::be_policy;
+        e_phnum = get_te16(&ehdri.e_phnum);
+    }
 };
 
 class PackLinuxElf32Le : public PackLinuxElf32
 {
     typedef PackLinuxElf32 super;
 protected:
-    PackLinuxElf32Le(InputFile *f) : super(f) { bele = &N_BELE_RTP::le_policy; }
-
-    virtual acc_uint64l_t get_native64(const void *b) const { return get_le64(b); }
-    virtual unsigned get_native32(const void *b) const { return get_le32(b); }
-    virtual unsigned get_native16(const void *b) const { return get_le16(b); }
-    virtual void set_native64(void *b, acc_uint64l_t v) const { set_le64(b, v); }
-    virtual void set_native32(void *b, unsigned v) const { set_le32(b, v); }
-    virtual void set_native16(void *b, unsigned v) const { set_le16(b, v); }
+    PackLinuxElf32Le(InputFile *f) : super(f) {
+        bele = &N_BELE_RTP::le_policy;
+        e_phnum = get_te16(&ehdri.e_phnum);
+    }
 };
 
 class PackLinuxElf64Le : public PackLinuxElf64
 {
     typedef PackLinuxElf64 super;
 protected:
-    PackLinuxElf64Le(InputFile *f) : super(f) { bele = &N_BELE_RTP::le_policy; }
-
-    virtual acc_uint64l_t get_native64(const void *b) const { return get_le64(b); }
-    virtual unsigned get_native32(const void *b) const { return get_le32(b); }
-    virtual unsigned get_native16(const void *b) const { return get_le16(b); }
-    virtual void set_native64(void *b, acc_uint64l_t v) const { set_le64(b, v); }
-    virtual void set_native32(void *b, unsigned v) const { set_le32(b, v); }
-    virtual void set_native16(void *b, unsigned v) const { set_le16(b, v); }
+    PackLinuxElf64Le(InputFile *f) : super(f) {
+        bele = &N_BELE_RTP::le_policy;
+        e_phnum = get_te16(&ehdri.e_phnum);
+    }
 };
 
+
 /*************************************************************************
 // linux/elf64amd
 **************************************************************************/
@@ -314,6 +336,7 @@ protected:
     virtual void defineSymbols(Filter const *);
 };
 
+
 /*************************************************************************
 // linux/elf32ppc
 **************************************************************************/
@@ -334,6 +357,7 @@ protected:
     virtual Linker* newLinker() const;
 };
 
+
 /*************************************************************************
 // linux/elf386
 **************************************************************************/
@@ -349,8 +373,6 @@ public:
     virtual const char *getFullName(const options_t *) const { return "i386-linux.elf"; }
     virtual const int *getFilters() const;
 
-    virtual void unpack(OutputFile *fo);
-
 protected:
     virtual void pack1(OutputFile *, Filter &);  // generate executable header
 
@@ -410,6 +432,7 @@ protected:
     );
 };
 
+
 /*************************************************************************
 // linux/elfarm
 **************************************************************************/
@@ -454,6 +477,25 @@ protected:
     virtual void defineSymbols(Filter const *);
 };
 
+class PackLinuxElf32mipseb : public PackLinuxElf32Be
+{
+    typedef PackLinuxElf32Be super;
+public:
+    PackLinuxElf32mipseb(InputFile *f);
+    virtual ~PackLinuxElf32mipseb();
+    virtual int getFormat() const { return UPX_F_LINUX_ELF32_MIPSEB; }
+    virtual const char *getName() const { return "linux/mipseb"; }
+    virtual const char *getFullName(const options_t *) const { return "mips-linux.elf"; }
+    virtual const int *getFilters() const;
+
+protected:
+    virtual Linker* newLinker() const;
+    virtual void pack1(OutputFile *, Filter &);  // generate executable header
+    virtual void buildLoader(const Filter *);
+    virtual void updateLoader(OutputFile *);
+    virtual void defineSymbols(Filter const *);
+};
+
 class PackLinuxElf32mipsel : public PackLinuxElf32Le
 {
     typedef PackLinuxElf32Le super;
@@ -466,7 +508,6 @@ public:
     virtual const int *getFilters() const;
 
 protected:
-    virtual const int *getCompressionMethods(int method, int level) const;
     virtual Linker* newLinker() const;
     virtual void pack1(OutputFile *, Filter &);  // generate executable header
     virtual void buildLoader(const Filter *);
@@ -474,6 +515,7 @@ protected:
     virtual void defineSymbols(Filter const *);
 };
 
+
 #endif /*} already included */
 
 
