@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -22,7 +22,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Markus F.X.J. Oberhumer              Laszlo Molnar
-   <mfx@users.sourceforge.net>          <ml1050@users.sourceforge.net>
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -64,15 +64,6 @@ protected:
 
     virtual void writePackHeader(OutputFile *fo);
 
-    // in order too share as much code as possible we introduce
-    // an endian abstraction here
-    virtual acc_uint64l_t get_native64(const void *) const = 0;
-    virtual unsigned get_native32(const void *) const = 0;
-    virtual unsigned get_native16(const void *) const = 0;
-    virtual void set_native64(void *, acc_uint64l_t) const = 0;
-    virtual void set_native32(void *, unsigned) const = 0;
-    virtual void set_native16(void *, unsigned) const = 0;
-
     virtual bool checkCompressionRatio(unsigned, unsigned) const;
 
 protected:
@@ -101,30 +92,28 @@ protected:
     unsigned b_len;  // total length of b_info blocks
 
     // must agree with stub/linux.hh
-    struct b_info { // 12-byte header before each compressed block
+    __packed_struct(b_info) // 12-byte header before each compressed block
         unsigned sz_unc;  // uncompressed_size
         unsigned sz_cpr;  //   compressed_size
         unsigned char b_method;  // compression algorithm
         unsigned char b_ftid;  // filter id
         unsigned char b_cto8;  // filter parameter
         unsigned char b_unused;
-    }
-    __attribute_packed;
-    struct l_info { // 12-byte trailer in header for loader
+    __packed_struct_end()
+
+    __packed_struct(l_info) // 12-byte trailer in header for loader
         LE32 l_checksum;
         LE32 l_magic;
         LE16 l_lsize;
         unsigned char l_version;
         unsigned char l_format;
-    }
-    __attribute_packed;
+    __packed_struct_end()
 
-    struct p_info { // 12-byte packed program header
+    __packed_struct(p_info) // 12-byte packed program header
         unsigned p_progid;
         unsigned p_filesize;
         unsigned p_blocksize;
-    }
-    __attribute_packed;
+    __packed_struct_end()
 
     struct l_info linfo;
 
@@ -142,39 +131,30 @@ class PackUnixBe32 : public PackUnix
     typedef PackUnix super;
 protected:
     PackUnixBe32(InputFile *f) : super(f) { bele = &N_BELE_RTP::be_policy; }
-    virtual acc_uint64l_t get_native64(const void *b) const { return get_be64(b); }
-    virtual unsigned get_native32(const void *b) const { return get_be32(b); }
-    virtual unsigned get_native16(const void *b) const { return get_be16(b); }
-    virtual void set_native64(void *b, acc_uint64l_t v) const { set_be64(b, v); }
-    virtual void set_native32(void *b, unsigned v) const { set_be32(b, v); }
-    virtual void set_native16(void *b, unsigned v) const { set_be16(b, v); }
 
     // must agree with stub/linux.hh
-    struct b_info { // 12-byte header before each compressed block
+    __packed_struct(b_info) // 12-byte header before each compressed block
         BE32 sz_unc;  // uncompressed_size
         BE32 sz_cpr;  //   compressed_size
         unsigned char b_method;  // compression algorithm
         unsigned char b_ftid;  // filter id
         unsigned char b_cto8;  // filter parameter
         unsigned char b_unused;
-    }
-    __attribute_packed;
-    struct l_info { // 12-byte trailer in header for loader
+    __packed_struct_end()
+
+    __packed_struct(l_info) // 12-byte trailer in header for loader
         BE32 l_checksum;
         BE32 l_magic;
         BE16 l_lsize;
         unsigned char l_version;
         unsigned char l_format;
-    }
-    __attribute_packed;
+    __packed_struct_end()
 
-    struct p_info { // 12-byte packed program header
+    __packed_struct(p_info) // 12-byte packed program header
         BE32 p_progid;
         BE32 p_filesize;
         BE32 p_blocksize;
-    }
-    __attribute_packed;
-
+    __packed_struct_end()
 };
 
 
@@ -183,39 +163,30 @@ class PackUnixLe32 : public PackUnix
     typedef PackUnix super;
 protected:
     PackUnixLe32(InputFile *f) : super(f) { bele = &N_BELE_RTP::le_policy; }
-    virtual acc_uint64l_t get_native64(const void *b) const { return get_le64(b); }
-    virtual unsigned get_native32(const void *b) const { return get_le32(b); }
-    virtual unsigned get_native16(const void *b) const { return get_le16(b); }
-    virtual void set_native64(void *b, acc_uint64l_t v) const { set_le64(b, v); }
-    virtual void set_native32(void *b, unsigned v) const { set_le32(b, v); }
-    virtual void set_native16(void *b, unsigned v) const { set_le16(b, v); }
 
     // must agree with stub/linux.hh
-    struct b_info { // 12-byte header before each compressed block
+    __packed_struct(b_info) // 12-byte header before each compressed block
         LE32 sz_unc;  // uncompressed_size
         LE32 sz_cpr;  //   compressed_size
         unsigned char b_method;  // compression algorithm
         unsigned char b_ftid;  // filter id
         unsigned char b_cto8;  // filter parameter
         unsigned char b_unused;
-    }
-    __attribute_packed;
-    struct l_info { // 12-byte trailer in header for loader
+    __packed_struct_end()
+
+    __packed_struct(l_info) // 12-byte trailer in header for loader
         LE32 l_checksum;
         LE32 l_magic;
         LE16 l_lsize;
         unsigned char l_version;
         unsigned char l_format;
-    }
-    __attribute_packed;
+    __packed_struct_end()
 
-    struct p_info { // 12-byte packed program header
+    __packed_struct(p_info) // 12-byte packed program header
         LE32 p_progid;
         LE32 p_filesize;
         LE32 p_blocksize;
-    }
-    __attribute_packed;
-
+    __packed_struct_end()
 };
 
 
