@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -21,8 +21,8 @@
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-   Markus F.X.J. Oberhumer   Laszlo Molnar
-   markus@oberhumer.com      ml1050@users.sourceforge.net
+   Markus F.X.J. Oberhumer              Laszlo Molnar
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -1121,6 +1121,11 @@ int Packer::getLoaderSize() const
     return size;
 }
 
+bool Packer::hasLoaderSection(const char *name) const
+{
+    void *section = linker->findSection(name, false);
+    return section != NULL;
+}
 
 int Packer::getLoaderSection(const char *name, int *slen) const
 {
