@@ -2,9 +2,9 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2009 Laszlo Molnar
-   Copyright (C) 2000-2009 John F. Reiser
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Laszlo Molnar
+   Copyright (C) 2000-2010 John F. Reiser
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -665,7 +665,7 @@ void *upx_main(
     // This is necessary for ET_DYN if|when we override a prelink address.
 
     entry = do_xmap((int)f_decompress, ehdr, &xi, av, &reloc, f_unf);
-    auxv_up(av, AT_ENTRY , entry);  // might not be necessary?
+    auxv_up(av, AT_ENTRY , entry);
 
   { // Map PT_INTERP program interpreter
     int j;
